/* broken start on constants/structs for the random question orderer
   Copyright (C) 1992-2000 Michigan State University

   The CAPA system is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License as
   published by the Free Software Foundation; either version 2 of the
   License, or (at your option) any later version.

   The CAPA system is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   General Public License for more details.

   You should have received a copy of the GNU General Public
   License along with the CAPA system; see the file COPYING.  If not,
   write to the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
   Boston, MA 02111-1307, USA.

   As a special exception, you have permission to link this program
   with the TtH/TtM library and distribute executables, as long as you
   follow the requirements of the GNU GPL in regard to all of the
   software in the executable aside from TtH/TtM.
*/

#define ALL_MIX     1
#define IMMOBILE    2
#define ALL_FIXED   3
#define FIRST_FIXED 4
#define BOTH_FIXED  5
#define LAST_FIXED  6
#define SINGLE      7
#define RANGE       8

typedef struct _RandGroup {
  int  type;
  int *list;
  int  length;
} RandGroup_t;

typedef struct _RandQO {
  int           num;
  RandGroup_t **groups;
  int          *used;
  int           length;
  int           lengthg;
} RandQO_t;


void rqo_finish();
void rqo_1spec();
void rqo_2spec();
void start_rqo_type(int type);
void append_rqo(Symbol*s);
void prefix_rqo(Symbol*s);

