while { ! [eof file0] } {
	set line [ gets file0 ]
	puts "name [lindex [split [lindex $line end] @] 0]"
	puts "password [lindex $line 3]"
	puts "title [string range $line 24 54]"
	puts "email [lindex $line end]"
	puts "my_vista 0"
	puts ""
}
