#!/usr/bin/wish
set exelocation "/usr/local/bin"
set imglocation "/usr/local/lib/CAPA"

set env(PATH) "/usr/local/teTeX/bin:$env(PATH):/usr/local/bin/X11:/usr/local/bin"
foreach program {quizzer grader manager} {
 image create photo $program.bm -file $imglocation/$program.ppm
 button .$program -image $program.bm -command "exec $exelocation/$program &"
 pack .$program
}
button .passwd -text Password -command "exec userpasswd &"
pack .passwd
button .xterm -text XTerm -command "exec rxvt &"
pack .xterm
button .gedit -text GEdit -command "exec gedit &"
pack .gedit
button .quit -text Quit -command "exit"
pack .quit
wm title . "Menu"
update idletasks
wm geometry . -0+20
