/* scorer initialization routines
   Copyright (C) 1992-2000 Michigan State University

   The CAPA system is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License as
   published by the Free Software Foundation; either version 2 of the
   License, or (at your option) any later version.

   The CAPA system is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   General Public License for more details.

   You should have received a copy of the GNU General Public
   License along with the CAPA system; see the file COPYING.  If not,
   write to the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
   Boston, MA 02111-1307, USA.

   As a special exception, you have permission to link this program
   with the TtH/TtM library and distribute executables, as long as you
   follow the requirements of the GNU GPL in regard to all of the
   software in the executable aside from TtH/TtM.
*/

/*
 * scorer.init.c
 * Guy Albertelli II 1998
 */
#include <stdio.h>
#include <sys/file.h>
#include <tk.h>
#include <stdlib.h>
#include "pProj/capaCommon.h"
#include <signal.h>
#include "scorer.h"
#include "manager.h"

Tcl_Interp * gInterp;

int Scorer_Init(Tcl_Interp *interp)
{
  char buf[BUFFER_SIZE];

  signal(SIGSEGV,signalHandler);
  signal(SIGILL,signalHandler);
  signal(SIGFPE,signalHandler);
  signal(SIGBUS,signalHandler);

  gInterp=interp;
  
  Tcl_CreateCommand(interp, "buildStudentList", buildStudentList,
		    (ClientData)NULL, (Tcl_CmdDeleteProc *)NULL);
  Tcl_CreateCommand(interp, "freeStudentList", freeStudentList,
		    (ClientData)NULL, (Tcl_CmdDeleteProc *)NULL);
  Tcl_CreateCommand(interp, "getAnswersFromSet", getAnswersFromSet,
		    (ClientData)NULL, (Tcl_CmdDeleteProc *)NULL);
  Tcl_CreateCommand(interp, "compareCapaID", compareCapaID,
		    (ClientData)NULL, (Tcl_CmdDeleteProc *)NULL);
  Tcl_CreateCommand(interp,"scorer_get_entry", scorer_get_entry,
		    (ClientData)NULL, (Tcl_CmdDeleteProc *)NULL);
  Tcl_CreateCommand(interp,"scorer_set_entry", scorer_set_entry,
		    (ClientData)NULL, (Tcl_CmdDeleteProc *)NULL);
  
  sprintf(buf,"%s.%s",MAJORVER,MANAGERMINORVER);
  Tcl_SetVar(interp,"gVer",buf,TCL_GLOBAL_ONLY);
  sprintf(buf,"Compiled on %s",COMPILE_DATE);
  Tcl_SetVar(interp,"gCompileDate",buf,TCL_GLOBAL_ONLY);
  
  return TCL_OK;
}

int Scorer_InitStandAlone(Tcl_Interp *interp)
{
  return Scorer_Init(interp);
}

int Scorer_SafeInit(Tcl_Interp *interp)
{
  return Scorer_Init(interp);
}

