/* Prototypes for all user accessible RANLIB routines 
   Copyright (C) 1992-2000 Michigan State University

   The CAPA system is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License as
   published by the Free Software Foundation; either version 2 of the
   License, or (at your option) any later version.

   The CAPA system is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   General Public License for more details.

   You should have received a copy of the GNU General Public
   License along with the CAPA system; see the file COPYING.  If not,
   write to the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
   Boston, MA 02111-1307, USA.

   As a special exception, you have permission to link this program
   with the TtH/TtM library and distribute executables, as long as you
   follow the requirements of the GNU GPL in regard to all of the
   software in the executable aside from TtH/TtM.
*/



#ifndef  _RAND_H
#define _RAND_H

#define  F_ARG(x)  x



float genbet  F_ARG((float aa,float bb));
float capa_gennor  F_ARG((double *num_d, float av,float sd));
#define EXTERN_F  

EXTERN_F void  advnst  F_ARG((long k));
/* EXTERN_F float genbet  F_ARG((float aa,float bb)); */
EXTERN_F float genchi  F_ARG((float df));
EXTERN_F float genexp  F_ARG((float av));
EXTERN_F float genf    F_ARG((float dfn, float dfd));
EXTERN_F float gengam  F_ARG((float a,float r));
EXTERN_F void genmn    F_ARG((float *parm,float *x,float *work));
EXTERN_F float gennch  F_ARG((float df,float xnonc));
EXTERN_F float gennf   F_ARG((float dfn, float dfd, float xnonc));
EXTERN_F float gennor  F_ARG((float av,float sd));
/* EXTERN_F float capa_gennor  F_ARG((double *num_d, float av,float sd)); */
EXTERN_F void genprm   F_ARG((long *iarray,int larray));
EXTERN_F float genunf  F_ARG((float low,float high));
EXTERN_F void getsd    F_ARG((long *iseed1,long *iseed2));
EXTERN_F void gscgn    F_ARG((long getset,long *g));
EXTERN_F long ignbin   F_ARG((long n,float pp));
EXTERN_F long ignlgi   F_ARG((void));
EXTERN_F long ignpoi   F_ARG((float mu));
EXTERN_F long ignuin   F_ARG((long low,long high));
EXTERN_F void initgn   F_ARG((long isdtyp));
EXTERN_F long mltmod   F_ARG((long a,long s,long m));
EXTERN_F void phrtsd   F_ARG((char* phrase,long* seed1,long* seed2));
EXTERN_F float ranf    F_ARG((void));
EXTERN_F float capa_ranf    F_ARG((void));
EXTERN_F void setall   F_ARG((long iseed1,long iseed2));
EXTERN_F void setant   F_ARG((long qvalue));
EXTERN_F void setgmn   F_ARG((float *meanv,float *covm,long p,float *parm));
EXTERN_F void setsd    F_ARG((long iseed1,long iseed2));
EXTERN_F float sexpo   F_ARG((void));
EXTERN_F float sgamma  F_ARG((float a));
EXTERN_F float snorm   F_ARG((void));

#endif
