--
Changes from 4.6.5 to 4.6.6 (distributed to capa2)

sourcing new common.tcl and utils.tcl in .c

Changed main to call Quizzer_Init, and compile in quizzer.init.c,
along with adding compile date info to global var gCompileDate

Changed version defs to be prepended with QUIZZER

--
Changes from 4.6.4 to 4.6.5

In answer only mode, student information is always placed atop the page.
(no matter the parsing mode)

Signals SEGV ILL FPE and BUS, are caught, global variable capaParsing
is used to detect whether these signals were thrown while parsing, if
so an error message as to the location we were parsing at when the
error occured is thrown up otherewise we just notify the user that
some action they performed has caused its death.

global variable gInterp exists in both standalone and tcl dependant modes

--
Changes from 4.6.3 to 4.6.4

changed capaRunLatex to return a 2 if gStopPrinting becomes a one

--
Changes from 4.6.2 to 4.6.3

added the target quizzer.2 and supporting targets to the makefile,
this target creates a standalone version of quizzer, it needs the
utility tcl2c-qz (a modified version of tcl2c), that takes quizzer.tcl
and generates a .c file.

added new file quizzer.init.c, basically is a stripped down main that
implements the new standalone functionality of quizzer

--
Changes from 4.6.1 to 4.6.2

capaPrintWholeClass removed (functionality moved to the Tcl code)

capaPrintSection removed (functionality moved to the Tcl code)

capaGetExisitingSections added, uses the capa call
capa_get_section_count and returns the info to the Tcl code

capaGetHeaderInfo checks capa_get_header's result code

--
Changes from 4.5.1 to 4.6.1 (Given to Ed to use on capa2)

now puts the student's name, number, PIN, etc at the top of an enscript preview

--
Changes from 1.3 to 4.5.1

Fixed latent bug in Devel code, sourced grader.tcl rather than quizzer.tcl

Fixed problem where if results directory didn't exist, it quietly fails.

Updated version number to new scheme
