/* broken beginning to reimplement the capalogin editor
   Copyright (C) 1992-2000 Michigan State University

   The CAPA system is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License as
   published by the Free Software Foundation; either version 2 of the
   License, or (at your option) any later version.

   The CAPA system is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   General Public License for more details.

   You should have received a copy of the GNU General Public
   License along with the CAPA system; see the file COPYING.  If not,
   write to the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
   Boston, MA 02111-1307, USA.

   As a special exception, you have permission to link this program
   with the TtH/TtM library and distribute executables, as long as you
   follow the requirements of the GNU GPL in regard to all of the
   software in the executable aside from TtH/TtM.
*/

/* gather subjective answers from student */

#define    BS    8
#define    DEL   127
#define    ESC   27

#define    COLON 58

#define EDIT_HEIGHT 21
#define EDIT_WIDTH 80

void init_editor(char*** sbuf_pp)
{
  int   ww=EDIT_WIDTH, hh=EDIT_HEIGHT,i;
  *sbuf_pp = (char **)capa_malloc(sizeof(char *),hh);
  for(i=0;i<hh;i++) {
    *sbuf_pp[i] = (char *)capa_malloc(sizeof(char)*ww+1,1);
  }
}

int handle_keystrokes_editor(char** sbuf_pp)
{
}

int editor(char*** sbuf_pp)
{
  init_editor(sbuf_pp);
  return handle_keystrokes_editor(*sbuf_pp);
}

void
answer_subjective(student_number,set,section,prob)
char  *student_number; 
int    set; 
int   *section;
int    prob;
{
  char **sbuf_pp;

  if (editor(&sbuf_pp)) {
    /*turn sbuf_pp into one string and store subjective*/
  }
}
