/* grader functions
   Copyright (C) 1992-2000 Michigan State University

   The CAPA system is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License as
   published by the Free Software Foundation; either version 2 of the
   License, or (at your option) any later version.

   The CAPA system is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   General Public License for more details.

   You should have received a copy of the GNU General Public
   License along with the CAPA system; see the file COPYING.  If not,
   write to the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
   Boston, MA 02111-1307, USA.

   As a special exception, you have permission to link this program
   with the TtH/TtM library and distribute executables, as long as you
   follow the requirements of the GNU GPL in regard to all of the
   software in the executable aside from TtH/TtM.
*/

/*
 * grader.h
 * Guy Albertelli II 1996
 */
#ifndef _GRADER_H_
#define _GRADER_H_

int capaGetHeader (ClientData clientdata, Tcl_Interp *interp, int argc, char *argv[]);
int capaGetStudents (ClientData clientdata, Tcl_Interp *interp, int argc, char *argv[]);
int capaFindSection (ClientData clientdata, Tcl_Interp *interp, int argc, char *argv[]);
int capaGetReportInfo (ClientData clientdata, Tcl_Interp *interp, int argc, char *argv[]);
int capaGetSetSummary (ClientData clientdata, Tcl_Interp *interp, int argc, char *argv[]);
int capaGetTermSummary (ClientData clientdata, Tcl_Interp *interp, int argc, char *argv[]);
int capaTclParse (ClientData clientdata, Tcl_Interp *interp, int argc, char *argv[]);
int capaSetupGAnswer (ClientData clientdata, Tcl_Interp *interp, int argc, char *argv[]);
int capaSaveGAnswer (ClientData clientdata, Tcl_Interp *interp, int argc, char *argv[]);
int capaExcuse (ClientData clientdata, Tcl_Interp *interp, int argc, char *argv[]);
int capaCreateSummary (ClientData clientdata, Tcl_Interp *interp, int argc, char *argv[]);
int capaGetQuestionTypes (ClientData clientdata, Tcl_Interp *interp, int argc, char *argv[]);
int capaSetHandGrade (ClientData clientdata, Tcl_Interp *interp, int argc, char *argv[]);
int capaSetScore (ClientData clientdata, Tcl_Interp *interp, int argc, char *argv[]);

#define NUM_SET_LIMIT 99
#define GET_ALL_SECTIONS 0
#define BUFFER_SIZE 2048

#define Q_PROBLEM '0'
#define Q_PROBLEM_AND_ANSWER '1'
#define Q_ANSWER '2'
#endif /* _GRADER_H_ */
