--
Changes from 1.3 to 1.4 (distributed to capa2 and the world, made Grader 5.0)

show # of Tcl commands executed through use of gCmd

modified info dialog to show gCompileDate var set in .c code

created a common.tcl in which common functions between grader and
quizzer are shared

--
Changes from 1.2 to 1.3

changed createSummaryReport to check whether createSummary returns an error

changed the "- y n E state" label in grading to be buttons that set
that students grade to all the same thing. (hangrade sets to min score
(0) on n max score on y and - or E on - or E


tk8.0 fixes, -transient False became -type tearoff

--
Changes from 1.1 to 1.2

removed all references to login counts (log Synopsis button etc.)

makes main list box show the found entry when found

stopped grader from realoading scores if that section is currently
loaded (new global gLoaded maintains this info)

