--
Changes from 2.10 to 3.1

fixed file saving so that blank lines are no longer inserted on to the end of files

fixed import so that the correct description of what is going to occur 
(webonlyline or stdline) is present

--
Changes from 2.9 to 2.10 (copied to ver5.0)

fixed closeDocument to not reset the gDir variable

--
Changes from 2.8 to 2.9 (distributed to capa2 and capa3)

can now find and replace in reference files

fixed find previous so that it actually worked (it never really did)

--
Changes from 2.7 to 2.8 (distributed to capa2)

fixed printMultiple sections so that it doesn't  throw up a "Sent print job"
after each section

throws error on a failed qzparse

changed the pickSections listbox to use extended rather than multiple mode

fixed it so printing can actually create the LaTeX output frame.

parseCapaConfig can survive bad capa.config files and alert the user to poorly defined lines (unbalenced quotes etc. . )

catch Destruction of windows ( the x in Win95 ) and do proper closing 

--
Changes from 2.6 to 2.7 (distributed to capa2)

modified info dialog to show gCompileDate var set in .c code

Created a common.tcl in which functions common between Grader and
Quizzer exist

--
Changes from 2.5 to 2.6

moved cut copy etc bindings to be in the creation of the main editing window

updated saveText to change gRefChanged($refNum) when a file is
succesfully saved

--
Changes from 2.4 to 2.5

select a defualt printing method in the print dialog

updated many toplevel window creations to have nice titles

can now cut, copy, paste, selectAll, save, save As, close reference
files, reference Files are checked if they need to be save.

added goto control box from reference file editor to main editing window

moved the clock to replace the usesless label "Assignment Information"

changed the bindings from the main controlWindow to be to .main rather
than all, also modified the Main Editing Window to be a child of .main

changed the Save and Print buttons on the preview and error windows to
read Save Output and Print Output

new proc, closeRefFile handles the closing of a refernece file, it
does not unset the variables associated with the window

textChange now takes an optional argument refNum, if specified it
updates gRefChanged($refNum) to 1

quit asks if you wish stop editing first, then if you wish to save files

quit now checks if if any of the gRefChanged entries that are set to
one still are mapped windows, and if they are it does askToSave on
each

askToSave takes an optional argument refNum, if specified it will try
saving a reference file with that number

cut, copy, paste and select All can take a refNum argument, in which
case it operates on the referece file with that number rather than the
main window

saveText now can take 1 2 or 3 arguments window saveAs(flag) refNum
only window is required, if saveAs is specified (then refNum must be
too) this controls whether or not it should query for filename, refNum
is the number of the reference file, it will also update the window
listing entry

Change the bindings on update{Ref}Location to be bound on Key Release
rather than KeyPress.

--
Changes from 2.3 to 2.4

new directory handling mechanism, rather than cding to the directory, 
just use tk_getOpenFile -initialdir option

changed the getProbValue call to check its return code, and modify the 
button to read "Import"

added more code to defeat to effects of the tk_getOpenFile bug, now set 
the focus to the tewwindow and generates a buttonpress-1 on the first 
character of the file.

--
Changes from 2.2 to 2.3

fixed how dvi creation works, when runLatex returns a 2 it means
gStopPrinting became a 1 at some point in time.

forgot to create gStopPrinting when starting up

tk_getOpenFile is buggy when creating a toplevel window after double
clicking on a file, giving it the parent arg and generating a button
press event for the parent makes it go away.

--
Changes from 2.1 to 2.2

stopped using File_select and are now using the tk_get{Open,Save}File options

changed the -transient option to -type (tk8.0 incompatability)

now using tcl/tk 8.0

--
Changes from 2.0 to 2.1

reads capa.config for the printer_option strings, if found it presents
a listbox allowing the user to selected among te different defintions
of the variable

changed the creation of Dvi text widget to wrap based on characters 
rather than words

added command printWholeClass that now prints a class a section at a
time using qzparse

checks if the directory is writable when opening a file and gives a
warning if it isn't

gets the qzparse command from the setting in capa.config

new printing option, print multiple Sections

printSection now uses qzparse to print a section

all of the normal printing commands now use the new getLprOptions

Gets the value of gCapaConfig(standardQuizzerHeader) from the
capa.config file, the listing in the capa.config file should be:
BeginStandardQuizzerHeader
headertext
EndStandardQuizzerHeader

Changed the getting of lprOptions to instead give the capa.config
possibilites as options.

Added proc parseCapaConfig, looks for the lprOneSided_command,
lprTwoSided_command, latex_command, and dvips_command values and uses
them in appropriate places

Save and Print buttons have been added to the Reference files

now detects when a user tries to open a directory rather than a file
for both openDocument and openReferenceFile

forgot to declare the globals in closeDocument

When picking a file to saveAs, if it is a directory, emit warning and
don't save

if gFile is "" when attempting to save, run the SaveAs code

getHeaderInfo is now checked if it returns an error and informs the
user if the setX.db file doesn't exist

added save and print buttons to parse error windows

added save and print buttons to the preview window (proc printText and
save Text )

--
Changes from 1.8 to 2.0 (Given to Ed to use on capa2)

added extra bindings so things like Alt-s wouldn't look like a
modification to the file

forgot to reset variables when a file got closed, like number of
questions and set number

added the import features that were in the Quizzer 5.0 test version to 4.6

new global gQuizTemp, and procedure changeMenuStatus control whether
or not quiztemp.* files can be created and therefore whther or not a
user can print and create .dvi files

openDocument now checks for old quiztemp files that can't be modified
(permissions) and warns user that create .dvi and printing won't work.

Actually deletes the quiztemp.* files it creates, should have been
doing that from day one.

Changed few remaining oks to Dismiss

changed the DBHeader warning message to use displayError instead of 
displayMessage

changed the error box to have a large "WARNING" in it. Maybe people
will notice it now.

if unable to open quiztemp.tex it warns the user about the problem

would crash if there was no question text in a question

in printSpecific and printStudent, was using
gStudentSelection(studentNumber) rather than
$gStudentSelection(studentNumber)

createDvi now checks if there are a different number of questions like
preview does.

now quizzer 4.6

--
Changes from 1.7 to 1.8

stopped yelling at the user for not specifying a print queue when they
attempted to cancel printing at the getLprOptions stage

added secret button to get value of auto_path variable, Shift-Alt-A

created global array gDir for remembering last directory that include,
import, references were from

createEditingWindow now has a return 

When creating a new document, save works correctly

Cut, Copy, Paste when there is no selection does not cause an error


