--
Changes from 4.6.2 to 4.6.3 (distributed to capa2)

sourcing new common.tcl and utils.tcl in .c

Changed main to call Quizzer_Init, and compile in quizzer.init.c,
along with adding compile date info to global var gCompileDate

changed version Defs to be prepended with GRADER

--
Changes from 4.6.1 to 4.6.2

capaCreateSummary now checks how many sets there are before
proceeding, to prevent it from bombing on non-existant sets.

fixed small memmory leak, forgot to free student list if an error
occurs while entering students into the listbox.

updates listbox only every 5th student

created grader.init.c from grader.c and added target grader.2 to the
makefile, can now compile standalone grader apps.

--
Changes from 4.5.1 to 4.6.1

deleted any functionality having to do with login counts. This
information is now being generated in capatools (Grading is much
faster now.)

no longer calculates login counts when loading a section for each person

looks for ./grader.tcl first

--
Changes from 1.1 to 4.5.1

Fixed problem in code with going to new decimal MAJORVER number.

Updated version number
