/* scantron control progam header
   Copyright (C) 1992-2000 Michigan State University

   The CAPA system is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License as
   published by the Free Software Foundation; either version 2 of the
   License, or (at your option) any later version.

   The CAPA system is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   General Public License for more details.

   You should have received a copy of the GNU General Public
   License along with the CAPA system; see the file COPYING.  If not,
   write to the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
   Boston, MA 02111-1307, USA.

   As a special exception, you have permission to link this program
   with the TtH/TtM library and distribute executables, as long as you
   follow the requirements of the GNU GPL in regard to all of the
   software in the executable aside from TtH/TtM.
*/

/*Get Form Errors*/
#define GFSUCCESS	100
#define GFFAILED	101
#define GFEOF		102
#define GFSPACES	103

/*Check Form Errors*/
#define ESTID		201
#define ECLASSL		202
#define EPIN		203
#define ECLASS		204

/*findPID Errors*/
#define ENONE		301
#define ENOONE		302

/*questions array size*/
#define MAXQUEST	40

/*grading stryles*/
#define KASHYMETHOD	1
#define MRTMETHOD	2
#define STRICT		3

typedef struct
{
	char answerPID[10];
	char questionPID[10];
	char class[9];
	char SetId[6];
	char PIN[7];
	char Answers[50][6];
} Student;

typedef struct
{
	char type;
	int leafs;
	int points;
} Question;

typedef struct
{
	char PID[MAX_STUDENT_NUMBER];
	int PIN;
} PIDPINlist;
